import asyncio

from aiohttp import web

from web_interface import handlers, auth

CAMERA_SWITCH_TIMEOUT = 10

async def camera_controller(request):
    auth.check_auth(request)
    status = 200
    data = None
    path = request.match_info.get('path')
    camera_controller = getattr(request.app['3dp_app'], 'camera_controller', None)
    if not camera_controller:
        status = 425
    elif request.method == 'GET':
        if path == 'current':
            data = camera_controller.current_camera_name
        elif path == 'enable':
            data = camera_controller.enabled
        elif path == 'types':
            data = list(camera_controller.CAMERA_MODULES.keys())
        else:
            raise web.HTTPNotFound()
    elif request.method == 'POST':
        try:
            post_data = await request.json()
        except:
            data = {'error': 'post without or with invalid json body'}
            status = 400
        else:
            if path == 'current':
                if isinstance(post_data, dict):
                    camera_name = post_data['name']
                elif isinstance(post_data, str):
                    camera_name = post_data
                if camera_name in camera_controller.CAMERA_MODULES.keys():
                    camera_switch_future = asyncio.get_event_loop().run_in_executor(
                            request.app['threads_pool'],
                            camera_controller.switch_camera, camera_name)
                    await asyncio.wait_for(camera_switch_future, CAMERA_SWITCH_TIMEOUT)
                else:
                    data = {'error': 'no such module: ' + str(camera_name)}
                    status = 400
            elif path == 'enable':
                if isinstance(post_data, dict):
                    enable = post_data['enable']
                elif isinstance(post_data, bool):
                    enable = post_data
                else:
                    raise web.HTTPBadRequest()
                if camera_controller.enabled == enable:
                    data = {'error': f'camera is already in got enable set to {enable}'}
                    status = 400
                else:
                    camera_controller.enabled = enable
                    if enable:
                        camera_start_future = asyncio.get_event_loop().run_in_executor(
                                request.app['threads_pool'],
                                camera_controller.start_camera_process)
                        await asyncio.wait_for(camera_start_future, CAMERA_SWITCH_TIMEOUT)
            else:
                raise web.HTTPNotFound()
    return web.json_response(data, status=status)
